-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use work.ImPack.all;
use work.vs6524.all;


entity ImageE is
  port (
    Button : in std_logic;                 -- Reset, low-aktiv
    Clk50 : in std_logic;                  -- Takt FPGA-Board

    PixCam   : in std_logic_vector(7 downto 0);  -- Daten von Kamera
    HSyncCam : in std_logic;               -- Sync horizontal von Kamera 
    VSyncCam : in std_logic;               -- Sync vertika von Kamera 
    PClk       : in std_logic;             -- Pixeltakt von Kamera

    R    : out std_logic_vector(7 downto 0);   -- Pixel rot
    G  : out std_logic_vector(7 downto 0);     -- Pixel gruen
    B   : out std_logic_vector(7 downto 0);    -- Pixel blau
    HSync : out std_logic;                     -- Sync horizontal fuer DAC
    VSync : out std_logic;                     -- Sync vertikal fuer DAC
    Clk25  : out std_logic;                    -- Takt fuer DAC

    SCL       : inout std_logic;          -- I2C Takt
    SDA         : inout std_logic;        -- I2C Daten
    init_ready  : out   std_logic;        -- Anzeige Kameraintialisierung
    cam_ce      : out   std_logic;        -- Freigabe Kameratakt
    cam_clk_out : out   std_logic;        -- Takt fuer Kamera
                                         
    Mode :out std_logic_vector(3 downto 0));                                        
end ImageE;


architecture ImageA of ImageE is    


-- Deklarationsteil part2s_a, Komponenten  
 
  component  ImProE IS
      PORT( 
      Button       : IN     std_logic;
      Clk12        : IN     std_logic;
      Reset        : IN     std_logic;
      PixR	   : IN     std_logic_vector (11 DOWNTO 0);
      WeA	   : OUT    std_logic_vector(0 downto 0);
      Mode         : OUT    std_logic_vector (3 DOWNTO 0);
      PixE	   : OUT    std_logic_vector (11 DOWNTO 0);
      ResetAll     : OUT    std_logic);
  end component;
  
  component ImWriteE 
  port (Reset      : in  std_logic;       -- Reset, low-aktiv
        PixCam     : in  std_logic_vector(7 downto 0);  -- Daten von Kamera
        HSyncCam   : in  std_logic;       -- Hsync von Kamera
        VSyncCam   : in  std_logic;       -- Vsync von Kamera
        PClk       : in  std_logic;       -- Pixelclock von Kamera
        ClkW       : out std_logic;       -- ClkW fuer Ram
        AddrW      : out std_logic_vector(CDepth-1 downto 0);   -- RAM-Adresse a
        PixW       : out std_logic_vector(CWidth-1 downto 0));  -- Daten Ram
  end component;

  component ramqvga                       
    port (
      clka  : in  std_logic;
      wea   : in  std_logic_vector(0 downto 0);
      addra : in  std_logic_vector(CDepth-1 downto 0);
      dina  : in  std_logic_vector(CWidth-1 downto 0);
      clkb  : in  std_logic;
      addrb : in  std_logic_vector(CDepth-1 downto 0);
      doutb : out std_logic_vector(CWidth-1 downto 0));
  end component;

  component ImReadE is
  port (
    Clk25    : in  std_logic;                            -- clock vga
    Reset    : in  std_logic;                            -- Reset, low-active
    PixR     : in  std_logic_vector(CWidth-1 downto 0);  -- ram data
    AddrR    : out std_logic_vector(CDepth-1 downto 0);  -- ram address
    R        : out std_logic_vector(7 downto 0);         -- pixel R value
    G        : out std_logic_vector(7 downto 0);         -- pixel G value
    B        : out std_logic_vector(7 downto 0);         -- pixel B value
    HSync    : out std_logic;                            -- horizontal sync
    VSync    : out std_logic);                           -- vertical sync
  end component;

  component clockgen                    -- Taktgererator
    port (
      U1_CLKIN_IN        : in  std_logic;
      U1_RST_IN          : in  std_logic;
      clk12              : out std_logic;
      Clk25              : out std_logic;
      U1_CLKDV_OUT       : out std_logic;
      U1_CLKFX_OUT       : out std_logic;
      U1_CLKIN_IBUFG_OUT : out std_logic;
      U1_CLK0_OUT        : out std_logic;
      U1_STATUS_OUT      : out std_logic_vector (7 downto 0);
      U2_CLKFX_OUT       : out std_logic;
      U2_CLK0_OUT        : out std_logic;
      U2_LOCKED_OUT      : out std_logic;
      U2_STATUS_OUT      : out std_logic_vector (7 downto 0));
  end component;

  component i2c_vs6524_init             --I2C-Kameracontroller
    port (
      clk         : in    std_logic;
      nReset      : in    std_logic;
      init_ready  : out   std_logic;
      Dout        : out   std_logic_vector(7 downto 0);
      error       : out   std_logic;
      SCL         : inout std_logic;
      SDA         : inout std_logic;
      cam_ce      : out   std_logic;
      cam_clk_out : out   std_logic;
      cam_clk_in  : in    std_logic);
  end component;

  signal ClkWS    : std_logic;
  signal WeaS     : std_logic_vector(0 downto 0);
  signal AddrWS   : std_logic_vector(CDepth-1 downto 0);
  signal PixWS    : std_logic_vector(CWidth-1 downto 0);
  signal AddrRS: std_logic_vector(CDepth-1 downto 0)  := (others => '0');
  signal PixRS: std_logic_vector(CWidth-1 downto 0);


  -- Signale Taktgenerator
  signal U1_CLKIN_INS       : std_logic;
  signal U1_RST_INS         : std_logic;
  signal clk12S             : std_logic;
  signal Clk25S             : std_logic;
  signal U1_CLKDV_OUTS      : std_logic;
  signal U1_CLKFX_OUTS      : std_logic;
  signal U1_CLKIN_IBUFG_OUTS: std_logic;
  signal U1_CLK0_OUTS       : std_logic;
  signal U1_STATUS_OUTS     : std_logic_vector (7 downto 0);
  signal U2_CLKFX_OUTS      : std_logic;
  signal U2_CLK0_OUTS       : std_logic;
  signal U2_LOCKED_OUTS     : std_logic;
  signal U2_STATUS_OUTS     : std_logic_vector (7 downto 0);

  -- Signale I2C-Kameracontroller
 
  signal DoutS       : std_logic_vector(7 downto 0);
  signal errorS      : std_logic;
  
  signal PixES, PixI : std_logic_vector(11 downto 0);
  signal ModeS   : std_logic_vector(3 downto 0);
  signal ButtonS            : std_logic;
  signal ResetS             : std_logic;
  signal ResetAllS: std_logic;
  
begin

  ImProL: ImProE
    port map (
        Button   => ButtonS,
        Clk12    => clk12S,
        Reset    => ResetS,
        Mode     => ModeS,
        PixE     => PixES,
        WeA      => WeaS,
        PixR     => PixRS,
		  ResetAll => ResetAllS);
  
  ImWriteL : ImWriteE
    port map (
      reset    => ResetAllS,
      PixCam   => PixCam,
      HSyncCam => HSyncCam,
      VSyncCam => VSyncCam,
      PClk     => PClk,
      ClkW     => ClkWS,
      AddrW    => AddrWS,
      PixW     => PixWS);

  ramqvga_l : ramqvga
    port map (
      clka  => ClkWS,
      wea   => WeaS, 
      addra => AddrWS,
      dina  => PixWS,
      clkb  => Clk25S,
      addrb => AddrRS,
      doutb => PixRS);

  ImReadL : ImReadE
    port map (
      Clk25    => Clk25S,
      Reset    => ResetAllS,
      PixR     => PixES,
      AddrR    => AddrRS,
      R        => R,
      G        => G,
      B        => B,
      HSync    => HSync,
      VSync    => VSync);

  clockgen_l : clockgen
    port map (
      U1_CLKIN_IN        => Clk50,
      U1_RST_IN          => U1_RST_INS,
      clk12              => clk12S,
      Clk25              => Clk25S,
      U1_CLKDV_OUT       => U1_CLKDV_OUTS,
      U1_CLKFX_OUT       => U1_CLKFX_OUTS,
      U1_CLKIN_IBUFG_OUT => U1_CLKIN_IBUFG_OUTS,
      U1_CLK0_OUT        => U1_CLK0_OUTS,
      U1_STATUS_OUT      => U1_STATUS_OUTS,
      U2_CLKFX_OUT       => U2_CLKFX_OUTS,
      U2_CLK0_OUT        => U2_CLK0_OUTS,
      U2_LOCKED_OUT      => U2_LOCKED_OUTS,
      U2_STATUS_OUT      => U2_STATUS_OUTS);

  i2c_vs6524_inil_l : i2c_vs6524_init
    port map (
      clk         => Clk25S,
      nReset      => ResetAllS,
      init_ready  => init_ready,
      Dout        => DoutS,
      error       => errorS,
      SCL         => SCL,
      SDA         => SDA,
      cam_ce      => cam_ce,
      cam_clk_out => cam_clk_out,
      cam_clk_in  => clk12S);

-- Zuweisungen
  Clk25 <= Clk25S;
--U1_RST_INS<= not ResetAllS;
  U1_RST_INS<= '0';  
  Mode <= ModeS;
--	WeaS<= "1";
  ResetS<= '1';
  ButtonS<= Button;
  
end ImageA;


